<?php
//
//                                                                         
//                                                                         
//                                                                         
//                                                                         
//                                                                         
//                                                                         
//                                                                         
//                                                                         
//        //           /       /                                           
//                                                                         
//                                                                         
//                                                                         
//                                                                         
//                                                                         
//                                                                         
//
//                                                      
//
//                                                                    
//           
//

require_once(INCLUDE_PATH."/common.php");

class DB_ibase extends DB_common
{
var $connection;
var $phptype, $dbsyntax;
var $autocommit = 1;
var $manip_query = array();

function DB_ibase()
{
$this->DB_common();
$this->phptype = 'ibase';
$this->dbsyntax = 'ibase';
$this->features = array(
'prepare' => true,
'pconnect' => true,
'transactions' => true,
'limit' => false
);
//                                                              
 //                                                   
 $this->errorcode_map = array(
-104 => DB_ERROR_SYNTAX,
-150 => DB_ERROR_ACCESS_VIOLATION,
-151 => DB_ERROR_ACCESS_VIOLATION,
-155 => DB_ERROR_NOSUCHTABLE,
-157 => DB_ERROR_NOSUCHFIELD,
-158 => DB_ERROR_VALUE_COUNT_ON_ROW,
-170 => DB_ERROR_MISMATCH,
-171 => DB_ERROR_MISMATCH,
-172 => DB_ERROR_INVALID,
-204 => DB_ERROR_INVALID,
-205 => DB_ERROR_NOSUCHFIELD,
-206 => DB_ERROR_NOSUCHFIELD,
-208 => DB_ERROR_INVALID,
-219 => DB_ERROR_NOSUCHTABLE,
-297 => DB_ERROR_CONSTRAINT,
-530 => DB_ERROR_CONSTRAINT,
-803 => DB_ERROR_CONSTRAINT,
-551 => DB_ERROR_ACCESS_VIOLATION,
-552 => DB_ERROR_ACCESS_VIOLATION,
-922 => DB_ERROR_NOSUCHDB,
-923 => DB_ERROR_CONNECT_FAILED,
-924 => DB_ERROR_CONNECT_FAILED
);
}

function connect($dsninfo, $persistent = false)
{
if (!DB::assertExtension('interbase')) {
return $this->raiseError(DB_ERROR_EXTENSION_NOT_FOUND);
}
$this->dsn = $dsninfo;
$user = $dsninfo['username'];
$pw   = $dsninfo['password'];
$dbhost = $dsninfo['hostspec'] ?
($dsninfo['hostspec'] . ':/' . $dsninfo['database']) :
$dsninfo['database'];

$connect_function = $persistent ? 'ibase_pconnect' : 'ibase_connect';

$params = array();
$params[] = $dbhost;
$params[] = !empty($user) ? $user : null;
$params[] = !empty($pw) ? $pw : null;
$params[] = isset($dsninfo['charset']) ? $dsninfo['charset'] : null;
$params[] = isset($dsninfo['buffers']) ? $dsninfo['buffers'] : null;
$params[] = isset($dsninfo['dialect']) ? $dsninfo['dialect'] : null;
$params[] = isset($dsninfo['role'])    ? $dsninfo['role'] : null;

/*
                                      
                                                           
                                     
                                                      
                            
                                               
                
                                                                  
         
        */
$conn = @call_user_func_array($connect_function, $params);
if (!$conn) {
return $this->ibaseRaiseError(DB_ERROR_CONNECT_FAILED);
}
$this->connection = $conn;
return DB_OK;
}

function disconnect()
{
$ret = @ibase_close($this->connection);
$this->connection = null;
return $ret;
}

function simpleQuery($query)
{
$ismanip = DB::isManip($query);
$this->last_query = $query;
$query = $this->modifyQuery($query);
$result = @ibase_query($this->connection, $query);
if (!$result) {
return $this->ibaseRaiseError();
}
if ($this->autocommit && $ismanip) {
ibase_commit($this->connection);
}
//                                                           
 //                                  
 return DB::isManip($query) ? DB_OK : $result;
}

//                       

/**
    *                                                         
    *                                                          
    *                                  
    *
    *                                      
    *                                                   
    *                                                   
    *
    *                                 
    *                                                          
    *                
    */

function modifyLimitQuery($query, $from, $count)
{
if ($this->dsn['dbsyntax'] == 'firebird') {
$from++; //                                                           
 $query = preg_replace('/^\s*select\s(.*)$/is',
"SELECT FIRST $count SKIP $from $1", $query);
}
return $query;
}

//    


//                 

/**
     *                                                                    
     *
     *                                     
     *
     *               
     *
     *                                                             
     */
function nextResult($result)
{
return false;
}

//    

function fetchInto($result, &$ar, $fetchmode, $rownum = null)
{
if ($rownum !== NULL) {
return $this->ibaseRaiseError(DB_ERROR_NOT_CAPABLE);
}
if ($fetchmode & DB_FETCHMODE_ASSOC) {
$ar = get_object_vars(ibase_fetch_object($result));
if ($ar && $this->options['optimize'] == 'portability') {
$ar = array_change_key_case($ar, CASE_LOWER);
}
} else {
$ar = ibase_fetch_row($result);
}
if (!$ar) {
if ($errmsg = ibase_errmsg()) {
return $this->ibaseRaiseError(null, $errmsg);
} else {
return null;
}
}
return DB_OK;
}

function freeResult()
{
if (is_resource($result)) {
return ibase_free_result($result);
}
if (!isset($this->prepare_tokens[(int)$result])) {
return false;
}
unset($this->prepare_tokens[(int)$result]);
unset($this->prepare_types[(int)$result]);
return true;
}

function freeQuery($query)
{
ibase_free_query($query);
return true;
}

function numCols($result)
{
$cols = ibase_num_fields($result);
if (!$cols) {
return $this->ibaseRaiseError();
}
return $cols;
}

function prepare($query)
{
$this->last_query = $query;
$query = $this->modifyQuery($query);
$stmt = ibase_prepare($query);
$this->manip_query[(int)$stmt] = DB::isManip($query);
return $stmt;
}

function execute($stmt, $data = false)
{
$result = ibase_execute($stmt, $data);
if (!$result) {
return $this->ibaseRaiseError();
}
if ($this->autocommit) {
ibase_commit($this->connection);
}
return DB::isManip($this->manip_query[(int)$stmt]) ? DB_OK : new DB_result($this, $result);
}

function autoCommit($onoff = false)
{
$this->autocommit = $onoff ? 1 : 0;
return DB_OK;
}

function commit()
{
return ibase_commit($this->connection);
}

function rollback($trans_number)
{
return ibase_rollback($this->connection, $trans_number);
}

function transactionInit($trans_args = 0)
{
return $trans_args ? ibase_trans($trans_args, $this->connection) : ibase_trans();
}

//             
 /**
     *                                  
     *
     *                                                    
     *                           
     *
     *               
     *                                                 
     *                                                               
     *                                          
     */
function nextId($seq_name, $ondemand = true)
{
$sqn = strtoupper(preg_replace('/[^a-z0-9_]/i', '_', $seq_name));
$repeat = 0;
do {
$this->pushErrorHandling(PEAR_ERROR_RETURN);
$result = $this->query("SELECT GEN_ID(${sqn}_SEQ, 1) FROM RDB\$GENERATORS"
." WHERE RDB\$GENERATOR_NAME='${sqn}_SEQ'");
$this->popErrorHandling();
if ($ondemand && DB::isError($result)) {
$repeat = 1;
$result = $this->createSequence($seq_name);
if (DB::isError($result)) {
return $result;
}
} else {
$repeat = 0;
}
} while ($repeat);
if (DB::isError($result)) {
return $result;
}
$arr = $result->fetchRow(DB_FETCHMODE_ORDERED);
$result->free();
return $arr[0];
}

//    
 //                     

/**
     *                    
     *
     *                                                 
     *                                                    
     *               
     */
function createSequence($seq_name)
{
$sqn = strtoupper(preg_replace('/[^a-z0-9_]/i', '_', $seq_name));
$this->pushErrorHandling(PEAR_ERROR_RETURN);
$result = $this->query("CREATE GENERATOR ${sqn}_SEQ");
$this->popErrorHandling();

return $result;
}

//    
 //                   

/**
     *                
     *
     *                                                 
     *                                                    
     *               
     */
function dropSequence($seq_name)
{
$sqn = strtoupper(preg_replace('/[^a-z0-9_]/i', '_', $seq_name));
return $this->query("DELETE FROM RDB\$GENERATORS WHERE RDB\$GENERATOR_NAME='${sqn}_SEQ'");
}

//    
 //                       

/**
      *                         
      *
      *                                                
      *                                                
      *
      *                                                                               
      *                                                               
      *                
      */
function _ibaseFieldFlags($field_name, $table_name)
{

$sql = 'SELECT  R.RDB$CONSTRAINT_TYPE CTYPE'
.' FROM  RDB$INDEX_SEGMENTS I'
.' JOIN  RDB$RELATION_CONSTRAINTS R ON I.RDB$INDEX_NAME=R.RDB$INDEX_NAME'
.' WHERE  I.RDB$FIELD_NAME=\''.$field_name.'\''
.' AND R.RDB$RELATION_NAME=\''.$table_name.'\'';
$result = ibase_query($this->connection, $sql);
if (empty($result)) {
return $this->ibaseRaiseError();
}
if ($obj = @ibase_fetch_object($result)) {
ibase_free_result($result);
if (isset($obj->CTYPE)  && trim($obj->CTYPE) == 'PRIMARY KEY') {
$flags = 'primary_key ';
}
if (isset($obj->CTYPE)  && trim($obj->CTYPE) == 'UNIQUE') {
$flags .= 'unique_key ';
}
}

$sql = 'SELECT  R.RDB$NULL_FLAG AS NFLAG,'
.' R.RDB$DEFAULT_SOURCE AS DSOURCE,'
.' F.RDB$FIELD_TYPE AS FTYPE,'
.' F.RDB$COMPUTED_SOURCE AS CSOURCE'
.' FROM  RDB$RELATION_FIELDS R '
.' JOIN  RDB$FIELDS F ON R.RDB$FIELD_SOURCE=F.RDB$FIELD_NAME'
.' WHERE  R.RDB$RELATION_NAME=\''.$table_name.'\''
.' AND  R.RDB$FIELD_NAME=\''.$field_name.'\'';
$result = ibase_query($this->connection, $sql);
if (empty($result)) {
return $this->ibaseRaiseError();
}
if ($obj = @ibase_fetch_object($result)) {
ibase_free_result($result);
if (isset($obj->NFLAG)) {
$flags .= 'not_null ';
}
if (isset($obj->DSOURCE)) {
$flags .= 'default ';
}
if (isset($obj->CSOURCE)) {
$flags .= 'computed ';
}
if (isset($obj->FTYPE)  && $obj->FTYPE == 261) {
$flags .= 'blob ';
}
}

return trim($flags);
}

//    
 //                

/**
      *                                                  
      *
      *                                                                    
      *
      *                                                                  
      *                                                                     
      *                                      
      *
      *                                                
      */
function tableInfo($result, $mode = null)
{
$count = 0;
$id    = 0;
$res   = array();

/*
          *                                                           
          *
          *                           
          *           
          *                           
          *                           
          *                           
          *                             
          *                            
          *
          *                             
          *           
          *                                            
          *                           
          *                           
          *                           
          *                             
          *                            
          *                                                           
          *                                                                
          *                                                      
          *
          *                                  
          *                                       
          *                                                        
          *                        
          *
          *                                                        
          *                                      *                   
          *                                   
          *
          *                                            
          *                                                       
          *                              *                          
          */

//                                                         
 //                          

if (is_string($result)) {
$id = ibase_query($this->connection,"SELECT * FROM $result");
if (empty($id)) {
return $this->ibaseRaiseError();
}
} else { //                                           
 $id = $result;
if (empty($id)) {
return $this->ibaseRaiseError();
}
}

$count = @ibase_num_fields($id);

//                                                                    
 if (empty($mode)) {

for ($i=0; $i<$count; $i++) {
$info = @ibase_field_info($id, $i);
$res[$i]['table'] = (is_string($result)) ? $result : '';
$res[$i]['name']  = $info['name'];
$res[$i]['type']  = $info['type'];
$res[$i]['len']   = $info['length'];
$res[$i]['flags'] = (is_string($result)) ? $this->_ibaseFieldFlags($info['name'], $result) : '';
}

} else { //     
 $res["num_fields"]= $count;

for ($i=0; $i<$count; $i++) {
$info = @ibase_field_info($id, $i);
$res[$i]['table'] = (is_string($result)) ? $result : '';
$res[$i]['name']  = $info['name'];
$res[$i]['type']  = $info['type'];
$res[$i]['len']   = $info['length'];
$res[$i]['flags'] = (is_string($result)) ? $this->_ibaseFieldFlags($info['name'], $result) : '';
if ($mode & DB_TABLEINFO_ORDER) {
$res['order'][$res[$i]['name']] = $i;
}
if ($mode & DB_TABLEINFO_ORDERTABLE) {
$res['ordertable'][$res[$i]['table']][$res[$i]['name']] = $i;
}
}
}

//                                                  
 if (is_resource($id)) {
ibase_free_result($id);
}
return $res;
}

//    
 //                      

/**
    *                                                  
    *                                                           
    *                                    
    */
function getSpecialQuery($type)
{
switch ($type) {
case 'tables':
default:
return null;
}
return $sql;
}

//    
 //                      

function ibaseRaiseError($errno = null, $errmsg = null)
{
if ($errmsg === null)
$errmsg = ibase_errmsg();
//                                                                     
 //                                                                   
 if (preg_match('/^([^0-9\-]+)([0-9\-]+)\s+(.*)$/', $errmsg, $m)) {
if ($errno === null) {
$ibase_errno = (int)$m[2];
//                                                                        
 //                                                       
 switch ($ibase_errno) {
case -204:
if (is_int(strpos($m[3], 'Table unknown'))) {
$errno = DB_ERROR_NOSUCHTABLE;
}
break;
default:
$errno = $this->errorCode($ibase_errno);
}
}
$errmsg = $m[2] . ' ' . $m[3];
}

return $this->raiseError($errno, null, null, $errmsg,
$this->last_query);
}

//    

}

/*
 *                 
 *             
 *                  
 *     
 */

?>
